<?php

	/*
	
	Plugin Name: WP-SimpleMeetingConfirmation
	Plugin URI: http://vuong.fr/myitblog/wp-simplemeetingconfirmation
	Description: Allows registered user to confirm if they will be present at a planned meeting and eventually add comments and number of participants with them.
	Version: 1.0
	Author: Frederic VUONG
	Author URI: http://vuong.fr/myitblog/about/
	License: GPL2
		
	*/

	/*
		
	Usage: Create a new post and add the following shortcode
	
	[SMC date="dateParameter"]
	
	You can optionally add the following parameters:  [SMC date="dateParameter" location="address" reqguests="true" description="a meeting description" displayresults="true"]
	
	- date= A date or any string that will be used as a unique identifier of the meeting [mandatory field]
	- reqguests= if set to "true", will show the fields to add a number of guests invited by the registered user [optional field]
	- description= if added, will add a short description to the meeting [optional field]
	- displayresults= if set to "true", will show the results in a table format [optional field]
		
	and also optionally, add some additional content with the SMC tags that will appear in the form table.
	
	Ex.: [SMC date="17/10/2010" location="Our Place" description="Housewarming party"]<img src="http://mysite.com/ourplace.jpg"[/SMC]
		
	TODO:
	- css file for messages
	- administration page + administration menu	
		- drop table
		- delete records
	
	*/

	// Global data for configuration
	global $plugin_table;
	global $plugin_name;
	global $thisDir;

	// Global data to store parameters from the short code
	global $date;
	global $reqguests;
	global $description;
	global $displayresults;
	
	// define the name of the table and the name of the plugin
	$plugin_table = 'WP_SMC';
	$plugin_name = 'simpleMeetingConfirmation';
	$thisDir = dirname( plugin_basename( __FILE__ ) );
	
	/*
	
	NAME: getCurrentUser()
	
	DESCRIPTION: Return currently logged user
	
	INPUTS:
	
	OUTPUTS:
		Current user object
		
	PROCESS:
		[1] Get current user information
		[2] Return object
		
	NOTES:		
	
	*/
	function getCurrentUser(){
	
		$current_user = wp_get_current_user();
		return $current_user;	
	}
	
	
	/*

	NAME: create_MeetingTable()
	
	DESCRIPTION: Create, in the WordPress database, the table that stores meeting confirmations
	
	INPUTS:
	
	OUTPUTS:
		1 if table created
		0 if table not created
		
	PROCESS:
		[1] Prepare query
		[2] Run query
		[2] Return query result
		
	NOTES:
	
	*/
	function create_MeetingTable(){
		
		global $wpdb;
		global $plugin_table;
		
		$sqlQuery = 'CREATE TABLE ' . $plugin_table . ' (id int not null auto_increment, userName varchar(50), meetingID varchar(20), answer varchar(10), nbParticipants int, comments varchar(255), PRIMARY KEY(id));';
		
		$result = mysql_query($sqlQuery);

		if (!$result) {
			die('create_MeetingTable() - Invalid query: ' . mysql_error());
		}
		
		// run the query and return result
		return $result;		
	}

	/* 

	NAME: drop_MeetingTable()
	
	DESCRIPTION: Drops, in the WordPress database, the table that stores meeting confirmations
	
	INPUTS:
	
	OUTPUTS:
		1 if table dropped
		0 if table not dropped
		
	PROCESS:
		[1] Prepare query
		[2] Run query
		[2] Return query result
		
	NOTES:

	*/
	function drop_MeetingTable(){
	
		global $wpdb;
		global $plugin_table;
		
		$sqlQuery = 'DROP TABLE ' . $plugin_table . ';';
		
		$result = mysql_query($sqlQuery);

		if (!$result) {
			die('drop_MeetingTable() - Invalid query: ' . mysql_error());
		}
		
		// run the query and return result
		return $result;		
	}

	/*  

	NAME: addRegisteredUserToMeeting($meetingID, $current_user, $answer, $nbParticipants, $comments)
	
	DESCRIPTION: Add a user to an event
	
	INPUTS:
		$meetingID [String] : The unique identifier of the meeting
		$current_user [String] : The current (registered) user
		$answer [String] : The answer provided by the current user
		$nbParticipants [int] : The number of participants invited by the current user
		$comments [String] : Comments provided by the current user
	
	OUTPUTS:
		1 if query run properly
		0 if issue in the query
		
	PROCESS:
		[1] Check parameters
		[2] Prepare query
		[3] Run query
		[4] Return query result
		
	NOTES:
		
	*/
	function addRegisteredUserToMeeting($meetingID, $current_user, $answer, $nbParticipants, $comments){
	
		global $wpdb;
		global $plugin_table;
		
		if ($current_user == null)
			$current_user = getCurrentUser()->user_login;
		if ($nbParticipants == '')
			$nbParticipants = 0;
		if ($answer == 'on')
			$answer = 'checked';
			
		$sqlQuery = 'INSERT INTO ' . $plugin_table . '(userName, meetingID, answer, nbParticipants, comments) VALUES("' . $current_user . '", "' . $meetingID . '", "' . $answer . '", ' . $nbParticipants .', "' . $comments . '");';
		$result = mysql_query($sqlQuery);

		if (!$result) {
			die('addRegisteredUserToMeeting() - Invalid query: ' . mysql_error());
		}
		
		// run the query and return result
		return $result;		
	}

	
	/*  
	
	NAME: updateRegisteredUserToMeeting($meetingID, $current_user, $answer, $nbParticipants, $comments)
	
	DESCRIPTION: Update user information in an event
	
	INPUTS:
		$meetingID [String] : The unique identifier of the meeting
		$current_user [String] : The current (registered) user
		$answer [String] : The answer provided by the current user
		$nbParticipants [int] : The number of participants invited by the current user
		$comments [String] : Comments provided by the current user
	
	OUTPUTS:
		1 if query run properly
		0 if issue in the query
		
	PROCESS:
		[1] Check parameters
		[2] Prepare query
		[3] Run query
		[4] Return query result
		
	NOTES:
	
	*/
	function updateRegisteredUserToMeeting($current_user, $meetingID, $answer, $nbParticipants, $comments){
	
		global $wpdb;
		global $plugin_table;
		
		if ($current_user == null)
			$current_user = getCurrentUser()->user_login;
		if ($nbParticipants == '')
			$nbParticipants = 0;
		if ($answer == 'on')
			$answer = 'checked';
		
		$sqlQuery = 'UPDATE ' . $plugin_table . ' SET answer = "' . $answer . '", nbParticipants = ' . $nbParticipants . ', comments = "' . $comments . '" WHERE userName = "' . $current_user . '" AND meetingID = "' . $meetingID . '";';
		
		$result = mysql_query($sqlQuery);
		
		if (!$result) {
			die('updateRegisteredUserToMeeting() - Invalid query: ' . mysql_error());
		}

		return $result;		
	}
	
	/*  

	NAME: deleteRegisteredUserToMeeting($meetingID, $current_user)
	
	DESCRIPTION: Delete an event or a user from an event
	
	INPUTS:
		$meetingID [String] : The unique identifier of the meeting
		$current_user [String] : A user or * (for all users)
	
	OUTPUTS:
		1 if query run properly
		0 if issue in the query
		
	PROCESS:
		[1] Check parameters
		[2] Prepare query
		[3] Run query
		[4] Return query result
		
	NOTES:
		This function is never called. It has been developped for the administration interface, allowing the administrator to
		delete all users from a meeting.
	
	*/
	
	function deleteRegisteredUserToMeeting($meetingID, $current_user){
	
		global $wpdb;
		global $plugin_table;

		// $current_user = *, all users from the meeting are deleted
		if ($current_user == '*') 
			$sqlQuery = 'DELETE FROM ' . $plugin_table . ' WHERE meetingID = "' . $meetingID . '";';
		else 
			$sqlQuery = 'DELETE FROM ' . $plugin_table . ' WHERE meetingID = "' . $meetingID . '" AND userName = "' . $current_user . '";';			
		
		$result = mysql_query($sqlQuery);
		
		if (!$result) {
			die('deleteRegisteredUserToMeeting() - Invalid query: ' . mysql_error());
		}

		return $result;	
	}
	

	/*

	NAME: activatePlugin()
	
	DESCRIPTION: Create the table in the database when the plugin is activated
	
	INPUTS:
	
	OUTPUTS:
		1 if table created
		0 if table not created
		
	PROCESS:
		[1] Call function create_MeetingTable() 
		
	NOTES:
		
	*/
	
	function activatePlugin(){

		$result;
	
		$result = create_MeetingTable();
		return $result;
	}
	
	/*
	
	NAME: deactivatePlugin()
	
	DESCRIPTION: 
	
	INPUTS:
	
	OUTPUTS:
		
	PROCESS:
		[1]  
		
	NOTES:
		For future options of deleting records or dropping the table
	 
	*/
	
	function deactivatePlugin(){	
	}

	
	/*
	
	NAME: loadShortCodeParameters($atts)
	
	DESCRIPTION: Load parameters from the shortcode
	
	INPUTS:
		$atts [Array] : the Array of parameters
		
	OUTPUTS:
		0: Default return
		
	PROCESS:
		[1] Read parameters
		[2] Store them in global variables
		
	NOTES:
		When no parameters are passed, empty strings will be store in the global variables.
		
	*/
	
	function loadShortCodeParameters($atts) {
		
		global $date;
		global $location;
		global $reqguests;
		global $description;
		global $displayresults;

		// read parameters or assign them null values
		extract(shortcode_atts(array(
		'date' => '',
		'location' => '',
		'reqguests' => '',
		'description' => '',
		'displayresults' => '',		
		), $atts));
		
		return 0;
	}
	
	
	
	/*
	
	NAME: main($atts, $content=null)
	
	DESCRIPTION: Main function
	
	INPUTS:
		$atts [Array] : The Array of parameters
		
	OUTPUTS:
		0: Default return
		
	PROCESS:
		[1] Call function to read parameters
		[2] Check if mandatory parameters are provided and if user is logged
		[3] Check if page is posted back
		[4] Insert or Update record in database
		[5] Load inserted/updated user data
		[6] Load form, pre-filled if existing data
		[7] Display results, if option is provided
		
		
	NOTES:
		When no parameters are passed or user is not logged, an error message will be displayed.
	
	*/
	
	function main($atts, $content=null){
	
		global $meta;
		global $date;
		global $location;
		global $reqguests;
		global $description;
		global $displayresults;
		global $plugin_name;
		global $thisDir;
		
		$row_data;
		
		// load custom variables
		loadShortCodeParameters($atts);
			
		// check if SMC_Date field was filled, if so, display meeting information
		if ($date == '')  {
			echo __('Please add at least date="xx/xx/xxxx" as a parameter of the shortcode. <BR/> Ex.: [SMC date="13/12/2010"]!', $plugin_name);
			return false;
		}

		// check if user is logged in
		if (!is_user_logged_in()){
			echo __('You must be logged in to use this widget!', $plugin_name);
			return false;
		}

		// check if page was posted back
		if ($_POST['hidUpdate'] == true) {
		
			// new record
			if(countRecords($date, getCurrentUser()->user_login, false, false) > 0) 			
				updateRegisteredUserToMeeting(getCurrentUser()->user_login, $date, $_POST['chkPresent'], $_POST['txtNbGuests'], $_POST['txtComments']);
			
			// update existing record
			else
				addRegisteredUserToMeeting($date, getCurrentUser()->user_login, $_POST['chkPresent'], $_POST['txtNbGuests'], $_POST['txtComments']);					
		}

		// load data into a record
		$row_data = loadUserData($date, getCurrentUser()->user_login);
		
?>

		<script src='<?php bloginfo('wpurl'); ?>/wp-content/plugins/wp-<?php echo $plugin_name ?>/simpleMeetingConfirmation.js' type = 'text/javascript'></script>
		
		<!-- Display meeting form -->
		<form name='frmSMCRegistration' method='POST' action=''>
			<table border='2' width="80%">
				<tbody>
					<tr>
						<th><?php echo __('Meeting:', $plugin_name) ?></th>
						<td><?php echo $date ?></td>
						
						<?php if($content != null){
						
							$i = 3; // for default fields (date,  Present, submit button)
							if ($description != '') $i++;
							if ($location != '') $i++;
							if ($reqguests != '') $i++;
							
							echo '<td align="center" rowspan="' . $i . '">' . $content . '</td>';
						}
						?>
						
					</tr>
					<?php 

						if ($description != ''){
							echo '<tr>';
								echo '<th>' . __('Description:', $plugin_name) . '</th>';
								echo '<td>' . $description . '</td>';
							echo '</tr>';
						}

						if ($location != ''){
							echo '<tr>';
								echo '<th>' . __('Location:', $plugin_name) . '</th>';
								echo '<td>' . $location . '</td>';
							echo '</tr>';
						}
											
					?>
					
					<tr>
						<th><?php echo __('Name:', $plugin_name) ?></th>
						<td><?php echo getCurrentUser()->first_name . ' ' . getCurrentUser()->last_name . '</td>' ?>
					</tr>
					<tr>
						<th><?php echo __('Present:', $plugin_name) ?></th>
						<td align="center"><input type='checkbox' name='chkPresent' <?php echo $row_data['answer'] ?>></td>
					</tr>
					<?php if ($reqguests == 'true'){
							echo '<tr>';
								echo '<th>' . __('Number of guests:', $plugin_name) . '</th>';
								echo '<td><input type="textbox" name="txtNbGuests" value="' . $row_data['nbParticipants'] . '"></td>';
							echo '</tr>';
						}
					?>
					<tr>
						<th><?php echo __('Comments:', $plugin_name) ?></th>				
						<td colspan='2'><textarea name='txtComments' rows='10' cols='20'><?php echo $row_data['comments'] ?></textarea></td>
					</tr>

					<tr>
						<td colspan='3' align='right'><input type='button' onClick='checkFields();document.forms["frmSMCRegistration"].submit();' name='cmdSubmit' value='<?php echo __("Valider", $plugin_name) ?>'></td>
					</tr>
				</tbody>
			</table>
			<input name='hidUpdate' type='hidden' value='true'>
		</form>			
			
<?php

		if ($displayresults == 'true') {
			displayRecords($date);			
		}
		
	}
		
	/*

	NAME: loadUserData($meetingID, $userName)
	
	DESCRIPTION: Load user data from the database
	
	INPUTS:
		$meetingID [String] : A meeting identifier
		$userName [String] : A user name
		
	OUTPUTS:
		$row_data: User/meeting information from the database in an array
		
	PROCESS:
		[1] Check parameters
		[2] Prepare query
		[3] Run query
		[4] Return result
		
	NOTES:
		
	*/	
	
	function loadUserData($meetingID, $userName){
	
		global $wpdb;
		global $plugin_table;
		$row_data;
		$result;

		if ($current_user == null)
			$current_user = getCurrentUser()->user_login;
		
		$sqlQuery = 'SELECT userName, meetingID, answer, nbParticipants, comments FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '" AND userName="' . $userName . '";';
		$result =  mysql_query($sqlQuery);	
		
		if (!$result) {
			die('loadUserData() - Invalid query: ' . mysql_error());
		}
		
		$row_data = mysql_fetch_array($result);
		
		return $row_data;		
	}
	
	/*

	NAME: countRecords($meetingID, $userName, $yesOnly, $guests)
	
	DESCRIPTION: Allows to count records from the database based on some criteria
	
	INPUTS:
		$meetingID [string] : A meeting identifier
		$userName [string] : A user name
		$yesOnly [bool] : Only where answer is equal to checked
		$guests [bool] : Only guests
		
	OUTPUTS:
		$row[0]: The number of records
		
	PROCESS:
		[1] Check parameters
		[2] Prepare query
		[3] Run query
		[4] Return result
		
	NOTES:
		
	*/
	
	function countRecords($meetingID, $userName, $yesOnly, $guests){
	
		global $wpdb;
		global $plugin_table;
		$result;

		if ($userName == null) 
			$sqlQuery = 'SELECT COUNT(*) FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '" ;';
		else 
			$sqlQuery = 'SELECT COUNT(*) FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '" AND userName = "' . $userName .'";';

		if ($yesOnly == true)
			$sqlQuery = 'SELECT COUNT(*) FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '" AND answer = "checked";';

		if ($guests == true)
			$sqlQuery = 'SELECT sum(nbParticipants) FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '";';
			
			
		$result = mysql_query($sqlQuery);

		if (!$result) {
			die('countRecords() - Invalid query: ' . mysql_error());
		}
		
		$row = mysql_fetch_array($result);
			
		return $row[0];
	
	}
	
	/*
	
	NAME: displayRecords($meetingID)
	
	DESCRIPTION: Displays records in an HTML table
	
	INPUTS:
		$meetingID [String] : A meeting identifier
		
	OUTPUTS:
		0: Default return
		
	PROCESS:
		[1] Prepare query
		[2] Run query
		[3] Display results in an HTML table
		
	NOTES:
	
	*/
	
	function displayRecords($meetingID){

		global $wpdb;
		global $plugin_table;
		global $reqguests;
		global $plugin_name;
		$resultData;

		$sqlQuery = 'SELECT * FROM ' . $plugin_table .' WHERE meetingID="' . $meetingID . '" ;';
		
		$resultData =  mysql_query($sqlQuery);		
		
		echo '<table>';
		echo '<th>' . __('Name', $plugin_name) . '</th>';
		echo '<th>' . __('Present', $plugin_name) .'</th>';
		if ($reqguests == 'true')
			echo '<th>' . __('Number of guests', $plugin_name) . '</th>';
		echo '<th>' . __('Comments', $plugin_name) . '</th>';
		
		while($row = mysql_fetch_array($resultData)){
			echo '<tr>';		
				echo '<td>' . $row['userName'] . '</td>';
				echo '<td>' ; 
					if($row['answer'] == 'checked') 
						echo __('Yes', $plugin_name);
					else 
						echo __('No', $plugin_name); 
				echo '</td>';
				if ($reqguests == 'true')
					echo '<td>' . $row['nbParticipants'] . '</td>';		
				echo '<td>' . $row['comments'] . '</td>';		
				
			echo '</tr>';		
		}

		echo '<tr>';
		echo '<td><b>' . __('Grand Total: ') . __('Yes', $plugin_name) .'</b></td>';
		echo '<td><b>' . countRecords($meetingID, null, true, false) . '/' . countRecords($meetingID, null, false, false) . '</b></td>';
		echo '<td><b>' . countRecords($meetingID, null, true, true) . '</b></td>';
		echo '<td><b>' . (countRecords($meetingID, null, true, false) + countRecords($meetingID, null, true, true)) . '</b></td>';
		echo '</tr>';

		echo '</table>';

		echo '<hr>';	

		return 0;
		
	}

	/*
	
	NAME: initialize()
	
	DESCRIPTION: Load the localisation files 
	
	INPUTS:
				
	OUTPUTS:
		0: Default return
		
	PROCESS:
		[1] Load localisation file
		
	NOTES:

	*/
	
	function initialize() {
	
		global $plugin_name;
		global $thisDir;
			
		load_plugin_textdomain( $plugin_name, false, $thisDir . '/languages' );
		
		return 0;
	}

	/*
	
	NAME: add_action('init', 'initialize');
	
	DESCRIPTION: Call the initialize function on init hook
	
	INPUTS:
		[1] hook name
		[2] function name
				
	OUTPUTS:
				
	PROCESS:
				
	NOTES:

	*/

	add_action( 'init', 'initialize' );

	/*
	
	NAME: add_shortcode('SMC', 'main');
	
	DESCRIPTION: Enable the shortcode in the post
	
	INPUTS:
		[1] Shortcode name
		[2] function name
				
	OUTPUTS:
				
	PROCESS:
				
	NOTES:

	*/
		
	add_shortcode('SMC', 'main');

	/*
	
	NAME: register_activation_hook(__FILE__, 'activatePlugin');
	
	DESCRIPTION: Call the activatePlugin function when plugin is activated
	
	INPUTS:
		[1] location of the function
		[2] function name
				
	OUTPUTS:
				
	PROCESS:
				
	NOTES:

	*/
	
	register_activation_hook(__FILE__, 'activatePlugin');
	
	/*
	
	NAME: register_deactivation_hook(__FILE__, 'deactivatePlugin');
	
	DESCRIPTION: Call the deactivatePlugin function when plugin is deactivated
	
	INPUTS:
		[1] location of the function
		[2] function name
				
	OUTPUTS:
				
	PROCESS:
				
	NOTES:

	*/
	register_deactivation_hook(__FILE__, 'deactivatePlugin');	
?>
